package info.textgrid.middleware;

import java.text.ParseException;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetRequestBuilder;
import org.elasticsearch.action.get.GetResponse;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * FIXME
 * 
 * @author Maximilian Brodhun SUB
 * @version 1.1
 * @since 17.02.2014
 */

public class RecordDelivererDC extends RecordDelivererAbstract {

  private String[] fields;
  private String[] workFields;

  private String formatField;
  private String formatToFilter;
  private String dateOfObjectCreation;
  private String relationToFurtherMetadataObject;
  private String repositoryObjectURIPrefix;
  private String identifierField;

  /*
   * Lists for the Dublin Core elements. Lists are needed since each DC fields is possible to occur
   * several times
   */

  private String[] contributorList;
  private String[] coverageList;
  private String[] creatorList;
  private String[] dateList;
  private String[] descriptionList;
  private String[] formatList;
  private String[] identifierList;
  private String[] languageList;
  private String[] publisherList;
  private String[] relationList;
  private String[] relationForWorkList;
  private String[] rightList;
  private String[] sourceList;
  private String[] subjectList;
  private String[] titleList;
  private String[] typeList;

  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordDelivererDC.class);

  private OAI_ESClient oaiEsClient;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordDelivererInterface#getRecordById(java.lang.String)
   */
  public GetRecordType getRecordById(String id) {

    GetRecordType getRecordType = new GetRecordType();
    RecordType record = new RecordType();
    String changedId = id;
    DublinCoreBuilder dublinCoreBuilder;

    if (id.startsWith(this.repositoryObjectURIPrefix)) {
      changedId = changedId.replace(this.repositoryObjectURIPrefix, "");
    }

    // TODO Must Index and Type be set here? Isn't it already set in beans.xml?
    GetRequestBuilder recordById =
        OAI_ESClient
            .getOaiESClient()
            .prepareGet()
            .setIndex(OAI_ESClient.getEsIndex())
            .setType(OAI_ESClient.getEsType())
            .setFields(this.fields)
            .setId(changedId);

    GetResponse esResultObject = recordById.execute().actionGet();

    if (esResultObject.isExists()) {

      String identifier;

      if (this.dariah == true) {
        dublinCoreBuilder = putContentIntoDCFieldLists(esResultObject);
        identifier = esResultObject.getField(this.identifierField).getValue().toString();
        String dateOfCreation = "NO_DATE_SET!";

        if (esResultObject.getField(this.dateOfObjectCreation) != null) {
          try {
            dateOfCreation = OAIPMHUtillities
                .convertDateFormat(
                    esResultObject.getField(this.dateOfObjectCreation).getValue().toString())
                .toXMLFormat();
          } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          } catch (DatatypeConfigurationException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
          record.setMetadata(dublinCoreBuilder.getDC());
        }

        setRecordHeader(dateOfCreation, identifier, record, dublinCoreBuilder);
      }

      if (this.textgrid == true && esResultObject.getField(this.formatField).getValue().toString()
          .equals(this.formatToFilter)) {

        String workUri =
            esResultObject.getField(this.relationToFurtherMetadataObject).getValue().toString();
        workUri = workUri.substring(this.repositoryObjectURIPrefix.length());
        dublinCoreBuilder = putContentIntoDCFieldLists(esResultObject, furtherDCElements(workUri));
        identifier = esResultObject.getField(this.identifierField).getValue().toString();
        String dateOfCreation =
            esResultObject.getField(this.dateOfObjectCreation).getValue().toString();
        setRecordHeader(dateOfCreation, identifier, record, dublinCoreBuilder);
        record.setMetadata(dublinCoreBuilder.getDC());
      }

      getRecordType.setRecord(record);

    } else {
      getRecordType = null;
    }

    return getRecordType;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * @param dateOfCreation
   * @param identifier
   * @param record
   * @param dbc
   * @return
   */
  private static RecordType setRecordHeader(String dateOfCreation, String identifier,
      RecordType record, DublinCoreBuilder dbc) {

    HeaderType recordHeader = new HeaderType();

    recordHeader.setIdentifier(identifier);
    recordHeader.setDatestamp(dateOfCreation);
    record.setMetadata(dbc.getDC());
    record.setHeader(recordHeader);

    return record;
  }

  /**
   * @param id
   * @return
   */
  private GetResponse furtherDCElements(String id) {

    if (!id.endsWith(".0")) {
      id = id.concat(".0");
    }

    GetRequestBuilder getWorkValues =
        OAI_ESClient.getOaiESClient()
            .prepareGet()
            .setIndex(OAI_ESClient.getEsIndex())
            .setType(OAI_ESClient.getEsType())
            .setId(id)
            .setFields(this.workFields);

    GetResponse responseWorkValues = getWorkValues.execute().actionGet();

    // FIXME MAX!!! DO WE NEED THST DATE???
    // FIXME MAX!!! DO WE NEED THST DATE???
    // FIXME MAX!!! DO WE NEED THST DATE???

    // try {
    // if (responseWorkValues.getField(this.dateOfObjectCreation) != null) {
    // this.dates.add(OAIPMHUtillities.convertDateFormat(
    // responseWorkValues.getField(this.dateOfObjectCreation).getValue().toString())
    // .toXMLFormat());
    // }
    // } catch (ParseException e) {
    // log.debug(e);
    // } catch (DatatypeConfigurationException e) {
    // log.debug(e);
    // }

    return responseWorkValues;
  }

  /**
   * @param responseWorkValues
   * @param relatedWorkObject
   */
  private DublinCoreBuilder putContentIntoDCFieldLists(GetResponse responseWorkValues,
      GetResponse relatedWorkObject) {

    DublinCoreBuilder result = new DublinCoreBuilder();

    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(
        DublinCoreFieldLoader.setContributor(responseWorkValues, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.setCoverage(responseWorkValues, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.setCreator(relatedWorkObject, this.creatorList));
    result.setDate(DublinCoreFieldLoader.setDate(relatedWorkObject, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.setDescription(relatedWorkObject, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.setFormat(responseWorkValues, this.formatList));
    result.setIdentifier(
        DublinCoreFieldLoader.setIdentifier(responseWorkValues, this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.setLanguage(responseWorkValues, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.setPublisher(responseWorkValues, this.publisherList));
    result.setRelation(
        DublinCoreFieldLoader.setRelationForWork(responseWorkValues, this.relationList));
    result.setRights(DublinCoreFieldLoader.setRights(responseWorkValues, this.rightList));
    result.setSource(DublinCoreFieldLoader.setSources(responseWorkValues, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.setSubject(relatedWorkObject, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.setTitle(responseWorkValues, this.titleList));
    result.setType(DublinCoreFieldLoader.setType(relatedWorkObject, this.typeList));

    return result;
  }

  /**
   * @param responseWorkValues
   */
  private DublinCoreBuilder putContentIntoDCFieldLists(GetResponse responseWorkValues) {

    DublinCoreBuilder result = new DublinCoreBuilder();

    // Set DublinCore lists with content from elastic search results due to configuration.
    result.setContributor(
        DublinCoreFieldLoader.setContributor(responseWorkValues, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.setCoverage(responseWorkValues, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.setCreator(responseWorkValues, this.creatorList));
    result.setDate(DublinCoreFieldLoader.setDate(responseWorkValues, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.setDescription(responseWorkValues, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.setFormat(responseWorkValues, this.formatList));
    result.setIdentifier(
        DublinCoreFieldLoader.setIdentifier(responseWorkValues, this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.setLanguage(responseWorkValues, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.setPublisher(responseWorkValues, this.publisherList));
    result.setRelation(
        DublinCoreFieldLoader.setRelation(responseWorkValues, this.relationList));
    result.setRights(DublinCoreFieldLoader.setRights(responseWorkValues, this.rightList));
    result.setSource(DublinCoreFieldLoader.setSources(responseWorkValues, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.setSubject(responseWorkValues, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.setTitle(responseWorkValues, this.titleList));
    result.setType(DublinCoreFieldLoader.setType(responseWorkValues, this.typeList));

    return result;
  }

  // **
  // GETTERS AND SETTERS
  // **

  /**
   * @param formatToFilter
   */
  public void setFormatToFilter(String formatToFilter) {
    this.formatToFilter = formatToFilter;
  }

  /**
   * @param formatField
   */
  public void setFormatField(String formatField) {
    this.formatField = formatField;
  }

  /**
   * @return
   */
  public String getDateOfObjectCreation() {
    return this.dateOfObjectCreation;
  }

  /**
   * @param dateOfObjectCreation
   */
  public void setDateOfObjectCreation(String dateOfObjectCreation) {
    this.dateOfObjectCreation = dateOfObjectCreation;
  }

  /**
   * @return
   */
  public String getRelationToFurtherMetadataObject() {
    return this.relationToFurtherMetadataObject;
  }

  /**
   * @param relationToFurtherMetadataObject
   */
  public void setRelationToFurtherMetadataObject(String relationToFurtherMetadataObject) {
    this.relationToFurtherMetadataObject = relationToFurtherMetadataObject;
  }

  /**
   * @return
   */
  public String getRepositoryObjectURIPrefix() {
    return this.repositoryObjectURIPrefix;
  }

  /**
   * @param repositoryObjectURIPrefix
   */
  public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
    this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierPrefix
   */
  public void setIdentifierField(String identifierPrefix) {
    this.identifierField = identifierPrefix;
  }

  /**
   * @param contributorList
   */
  public void setContributor(String[] contributorList) {
    this.contributorList = contributorList;
  }

  /**
   * @return
   */
  public String[] getContributor() {
    return this.contributorList;
  }

  /**
   * @param coveragesList
   */
  public void setCoverage(String[] coveragesList) {
    this.coverageList = coveragesList;
  }

  /**
   * @param creatorsList
   */
  public void setCreator(String[] creatorsList) {
    this.creatorList = creatorsList;
  }

  /**
   * @param datesList
   */
  public void setDates(String[] datesList) {
    this.dateList = datesList;
  }

  /**
   * @param descriptionsList
   */
  public void setDescriptions(String[] descriptionsList) {
    this.descriptionList = descriptionsList;
  }

  /**
   * @param formatsList
   */
  public void setFormats(String[] formatsList) {
    this.formatList = formatsList;
  }

  /**
   * @param identifiersList
   */
  public void setIdentifiers(String[] identifiersList) {
    this.identifierList = identifiersList;
  }

  /**
   * @param languagesList
   */
  public void setLanguages(String[] languagesList) {
    this.languageList = languagesList;
  }

  /**
   * @param publishersList
   */
  public void setPublishers(String[] publishersList) {
    this.publisherList = publishersList;
  }

  /**
   * @param relationsList
   */
  public void setRelations(String[] relationsList) {
    this.relationList = relationsList;
  }

  /**
   * @param relationsList
   */
  public void setRelationsForWork(String[] relationsList) {
    this.relationForWorkList = relationsList;
  }

  /**
   * @param rightsList
   */
  public void setRights(String[] rightsList) {
    this.rightList = rightsList;
  }

  /**
   * @param sourcesList
   */
  public void setSources(String[] sourcesList) {
    this.sourceList = sourcesList;
  }

  /**
   * @param subjectsList
   */
  public void setSubjects(String[] subjectsList) {
    this.subjectList = subjectsList;
  }

  /**
   * @param titlesList
   */
  public void setTitles(String[] titlesList) {
    this.titleList = titlesList;
  }

  /**
   * @param typesList
   */
  public void setTypes(String[] typesList) {
    this.typeList = typesList;
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

  /**
   * @return
   */
  public String[] getWorkFields() {
    return this.workFields;
  }

  /**
   * @param workFields
   */
  public void setWorkFields(String[] workFields) {
    this.workFields = workFields;
  }

  /**
   * @param fields
   */
  public void setFields(String[] fields) {
    this.fields = fields;
  }

  /**
   * @return
   */
  public String[] getFields() {
    return this.fields;
  }

}
